/*
Copyright (C) 2012 Taisuke Kobayashi
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.io.File;
import java.io.FileWriter;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import javax.swing.JFrame;

public class ServerCanvas extends BaseCanvas {

  private final int Ulw = 320, Ulp = 2, Uln = 18;
  private int currentUserIdIndex = 0;
  private String[] userList = null;
  private String[] helpStringNormal = {
    "F1        ʃLv` ŏ",
    "Shift+F1  ʃLv` ŏ",
    "F2        5bɉʃLv` ŏ",
    "Shift+F2  5bɉʃLv` ŏ",
    "F3        10bɉʃLv` ŏ",
    "Shift+F3  10bɉʃLv` ŏ",
    "SPACE     wIʃw",
    "BACKSPACE MLZ / Sw։ʑM"
  };
  private String[] helpStringUserId = {
    "F12       wv\ On/Off",
    "F1        ʃLv` ŏ",
    "Shift+F1  ʃLv` ŏ",
    "SPACE     摜\ʃw߂",
    "BACKSPACE wʎM",
    "J[\  ړ&I",
    "NbN  ړ&I",
    "ESC       oȃt@C"
  };

  public ServerCanvas(Snapshot _snapshot) throws Exception{
    super(_snapshot);
  }

  public void capture(boolean iconifiedFlag, int msec){
    if(iconifiedFlag){
      snapshot.setExtendedState(JFrame.ICONIFIED);
      C.sleep(100);
    }
    if(msec > 0)
      C.sleep(msec);
    snapshot.server.captureAndSave();
    lastImage();
    drawMode = DRAW_FILE;
    snapshot.setExtendedState(JFrame.MAXIMIZED_BOTH);
  }

  String userInfo(int mode, int index){
    if(userList == null)
      return "";
    String ret = "";
    String[] sa = userList[index].split(ClientList.FIELD_SEP);
    if(sa[0].length() < 3) ret += sa[0];
    else ret += sa[0].substring(1, sa[0].length() -1);
    switch(mode){
    case 0:
      for(int i = 1; i < sa.length; i++)
        ret += ClientList.FIELD_SEP + sa[i];
      break;
    case 1:
      if(sa.length > 1){
        ret += " ";
        switch(sa[1].length()){
        case 0: ret += "@@@"; break;
        case 1: ret += sa[1] + "@@"; break;
        case 2: ret += sa[1] + "@"; break;
        default: ret += sa[1].substring(0, 3); break;
        }
      }
      if(sa.length > 5) ret += " " + sa[5];
      if(sa.length > 6) ret += " " + sa[sa.length -1];
      break;
    case 2:
      ret = sa[0];
      break;
    case 3:
      ret = sa[sa.length -1];
      ret = ret.substring(ret.length() -1);
      break;
    }
    return ret;
  }

  void paintUserId(Graphics g){
    FontMetrics fm = g.getFontMetrics();
    int fh = fm.getHeight(), fd = fm.getDescent();
    snapshot.setTitle("wI");
    String s = "" + userList.length + " l ";
    g.drawString(s, Ulp, fh + Ulp - fd);
    g.drawString(userInfo(0, currentUserIdIndex), Ulp + fm.stringWidth(s), fh + Ulp - fd);
    for(int i = 0; i < userList.length; i++){
      if(userInfo(3, i).charAt(0) == TcpBase.MESSAGE_TYPE_REMOTE_DOWN) g.setColor(Color.RED);
      g.drawString(userInfo(1, i), (i / Uln) * (Ulw + Ulp *2) + Ulp, ((i % Uln) +1 +1)  * (fh + Ulp *2) - fd - Ulp);
      g.setColor(Color.WHITE);
    }
    g.drawRect((currentUserIdIndex / Uln) * (Ulw + Ulp *2) + Ulp,
               (currentUserIdIndex % Uln +1) * (fh + Ulp *2) + Ulp, Ulw - Ulp *2, fh - Ulp *2);
    if(helpFlag)
      drawHelp(g);
  }

  public void mousePressedUserId(){
    if((userList == null) || (font == null))
      return;
    int h = getFontMetrics(font).getHeight() + Ulp *2;
    if(mouseY >= h){
      int i = (mouseX / Ulw) * Uln + (mouseY - h) / h;
      if(i < userList.length){
        currentUserIdIndex = i;
        repaint();
      }
    }
  }

  public void keyReleasedUserId(int key, int modifiers){
    int len = userList.length;
    switch(key){
    case KeyEvent.VK_F1: capture(modifiers == KeyEvent.SHIFT_DOWN_MASK, 0); break;
    case KeyEvent.VK_F12: helpFlag = !helpFlag; break;
    case KeyEvent.VK_BACK_SPACE:
      snapshot.server.requestSendImage(userInfo(2, currentUserIdIndex));
      drawMode = DRAW_NET;
      // break͌̈ӂɂĂȂB
    case KeyEvent.VK_SPACE:
      menuMode = MENU_NORMAL;
      canvasClearFlag = false;
      break;
    case KeyEvent.VK_ESCAPE:
      if(C.confirm("{ɏoȃt@Cɏ݂܂H"))
        writeUserListFile();
      break;
    case KeyEvent.VK_UP:
      currentUserIdIndex--;
      if(currentUserIdIndex < 0)
        currentUserIdIndex = len -1;
      break;
    case KeyEvent.VK_DOWN:
      currentUserIdIndex++;
      if(currentUserIdIndex >= len)
        currentUserIdIndex = 0;
      break;
    case KeyEvent.VK_LEFT:
      currentUserIdIndex -= Uln;
      if(currentUserIdIndex < 0){
        currentUserIdIndex += (len / Uln +1) * Uln;
        if(currentUserIdIndex >= len)
          currentUserIdIndex = len -1;
        if(currentUserIdIndex < 0)
          currentUserIdIndex = 0;
      }
      break;
    case KeyEvent.VK_RIGHT:
      currentUserIdIndex += Uln;
      if(currentUserIdIndex >= len){
        currentUserIdIndex -= (len / Uln +1) * Uln;
        if(currentUserIdIndex >= len)
          currentUserIdIndex = len -1;
        if(currentUserIdIndex < 0)
          currentUserIdIndex = 0;
      }
      break;
    }
    repaint();
  }

  public void keyReleasedNormal(int key, int modifiers){
    switch(key){
    case KeyEvent.VK_F1: capture(modifiers == KeyEvent.SHIFT_DOWN_MASK, 0); repaint(); break;
    case KeyEvent.VK_F2: capture(modifiers == KeyEvent.SHIFT_DOWN_MASK, 5000); repaint(); break;
    case KeyEvent.VK_F3: capture(modifiers == KeyEvent.SHIFT_DOWN_MASK, 10000); repaint(); break;
    case KeyEvent.VK_SPACE:
      userList = snapshot.server.clientList.getUserList();
      if(userList == null){
        C.info("[Uo^Ă܂B");
        break;
      }
      menuMode = MENU_USERID;
      canvasClearFlag = true;
      repaint();
      break;
    case KeyEvent.VK_BACK_SPACE:
      if(snapshot.server.getSendImageLoopFlag()){
        snapshot.server.cancelSendImage();
        C.info("Swւ̉ʑMI܂B");
      }else if(snapshot.server.getRequestSendImageFlag()){
        snapshot.server.cancelSendImage();
        C.info("w̉ʎMI܂B");
      }else{
        if(C.confirm("Swɉʂ𑗐M܂H"))
          snapshot.server.sendImage();
      }
      break;
    case KeyEvent.VK_ESCAPE:
      if(C.confirm("{ɏI܂H")){
        if(C.confirm("IƏoȏ͔j܂B" + C.cr
                     + "oȃt@Cɏ݂܂H")){
          writeUserListFile();
        }
        C.d("ServerCanvas: KeyReleasedNormal: VK_ESCAPE: end");
        C.exit();
      }
      break;
    default: super.keyReleasedNormal(key, modifiers); break;
    }
  }

  void drawHelp(Graphics g){
    switch(menuMode){
    case MENU_NORMAL: drawHelpNormal(g); break;
    case MENU_USERID: drawHelpUserId(g); break;
    }
  }

  void drawHelpUserId(Graphics g){
    FontMetrics fm = g.getFontMetrics();
    int h = fm.getHeight() + LINE_PADDING, a = fm.getAscent(), i = 0;
    for(i = 0; i < helpStringUserId.length; i++){
      String s = helpStringUserId[i];
      int y = (i +1) * h + Y_PADDING;
      g.clearRect(X_PADDING, y - a, HELP_WIDTH, h);
      g.drawString(s, X_PADDING, y);
    }
  }

  void drawHelpNormal(Graphics g){
    FontMetrics fm = g.getFontMetrics();
    int h = fm.getHeight() + LINE_PADDING, a = fm.getAscent(), i = 0;
    for(i = 0; i < helpString.length; i++){
      String s = helpString[i];
      int y = (i +1) * h + Y_PADDING;
      g.clearRect(X_PADDING, y - a, HELP_WIDTH, h);
      g.drawString(s, X_PADDING, y);
    }
    for(i = 0; i < helpStringNormal.length; i++){
      String s = helpStringNormal[i];
      int y = (i +1) * h + Y_PADDING;
      g.clearRect(X_PADDING + HELP_WIDTH, y - a, HELP_WIDTH, h);
      g.drawString(s, X_PADDING + HELP_WIDTH, y);
    }
  }

  void writeUserListFile(){
    userList = snapshot.server.clientList.getUserList();
    if(userList == null){
      C.info("[Uo^Ă܂B");
      return;
    }
    String path = snapshot.ini.getShussekiFilePath();
    try{
      File f = new File(path);
      f.setWritable(true);
      FileWriter fw = new FileWriter(f, true);
      String ts = C.getCustomDateTimeString(C.CUSTOM_DATE_TIME_COMMA) + ",", s = "";
      for(int i = 0; i < userList.length; i++){
        s = ts + userList[i] + C.cr;
        fw.write(s, 0, s.length());
      }
      fw.close();
      f.setWritable(false);
    }catch(Exception e){
      C.e("ServerCanvas: writeUserListFile: 1: " + e.toString());
      C.info("t@C " + path + " ɐɏ݂ł܂łB");
      return;
    }
    try{
      Runtime.getRuntime().exec("notepad.exe " + path);
    }catch(Exception e){
      C.e("ServerCanvas: writeUserListFile: 2: " + e.toString());
      C.info("notepad.exeNł܂łB");
    }
  }

}
