/*
Copyright (C) 2012 Taisuke Kobayashi
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;

public class Snapshot extends JFrame implements KeyListener {
  public static void main(String[] args){ new Snapshot(); }

  final String INI_FILE_NAME = "snapshot.ini";
  final String EXEC_MODE_SERVER = "server";
  static final String IMAGE_TYPE = "png";
  static final String PATH_FORMAT = "%s\\%s%s." + IMAGE_TYPE;
  private boolean serverFlag = false;
  String userId = null;
  Ini ini = null;
  Toolkit toolkit = null;
  Robot robot = null;
  Rectangle windowSize = null;
  Server server = null;
  Client client = null;
  BufferedImage captureImage = null;

  public Snapshot(){
    try{
      userId = System.getenv("USERNAME");
      ini = new Ini(INI_FILE_NAME);
      C.openLogFile(ini.getLogFilePath());
      C.log("start: " + userId);
      if(ini.getExecMode().equals(EXEC_MODE_SERVER))
        serverFlag = true;
      toolkit = Toolkit.getDefaultToolkit();
      robot = new Robot();
      windowSize = new Rectangle(toolkit.getScreenSize());
      setTitle("snapshot");
      setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
      int w = 300, h = 100, x = (int)windowSize.getWidth() - w, y = (int)windowSize.getHeight() - h -50;
      setBounds(x, y, w, h);
      if(serverFlag){
        server = new Server(this);
        add(server.getServerCanvas());
      }else{
        try{
          ini.readUnicastIpPortFileForClient();
        }catch(Exception e){
          C.e("Snapshot: Snapshot: " + e.toString());
        }
        client = new Client(this);
        add(client.getBaseCanvas());
      }
      addKeyListener(this);
      setVisible(true);
    }catch(Exception e){
      C.error("Snapshot: Snapshot:" + C.cr + e.toString());
      C.exit();
    }
  }

  public void capture() throws Exception{
    try{
      captureImage = robot.createScreenCapture(windowSize);
    }catch(Exception e){
      C.throwException("Snapshot: capture:", e);
    }
  }

  public void keyReleased(KeyEvent e){
    int key = e.getKeyCode(), modifiers = e.getModifiersEx();
    if(serverFlag)
      server.getServerCanvas().keyReleased(key, modifiers);
    else
      client.getBaseCanvas().keyReleased(key, modifiers);
  }
  public void keyTyped(KeyEvent e){}
  public void keyPressed(KeyEvent e){}

}
